﻿using C1.WPF.ExpressionEditor;
using C1.WPF.FlexGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml.Linq;

namespace ExpressionEditor_FlexGrid
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            List<Product> items = Product.GetData(200);
            flexgrid.ItemsSource = items;

            C1ExpressionEditor c1ExpressionEditor1 = new C1ExpressionEditor();
            c1ExpressionEditor1.Expression = "[Price]*0.95";
            flexgrid.ExpressionEditors.Add("CustomField1", c1ExpressionEditor1);
            c1ExpressionEditor1.Evaluate();

            flexgrid.CellFactory = new ImageCellFactory();

            flexgrid.BeginningEdit += Flexgrid_BeginningEdit;
            expressionEditor.OkClick += ExpressionEditor_OkClick;
            expressionEditor.CancelClick += ExpressionEditor_CancelClick;
        }
        string field = "";
        private void ExpressionEditor_CancelClick(object sender, EventArgs e)
        {
            if (field != string.Empty)
                if (flexgrid != null && flexgrid.ExpressionEditors.Contains(field))
                    expressionEditor.Expression = flexgrid.ExpressionEditors[field].Expression;
            popup.IsOpen = false;
        }
        private void ExpressionEditor_OkClick(object sender, EventArgs e)
        {
            if(field!=string.Empty)
                if (flexgrid != null && flexgrid.ExpressionEditors.Contains(field))
                    flexgrid.ExpressionEditors[field].Expression = expressionEditor.Expression;
            popup.IsOpen = false;
        }
        private void Flexgrid_BeginningEdit(object sender, C1.WPF.FlexGrid.CellEditEventArgs e)
        {
            if(e.Column==7)
            {
                popup.IsOpen = true;
                field = flexgrid.Columns[e.Column].ColumnName;
                if (flexgrid != null && flexgrid.ExpressionEditors.Contains(field))
                {
                    expressionEditor.DataSource = flexgrid.ExpressionEditors[field].DataSource;
                    expressionEditor.Expression = flexgrid.ExpressionEditors[field].Expression;
                }
            }
        }
    }
    public class Product : BaseObject
    {
        string _name, _color, _line;
        double _price, _cost;
        DateTime _date;
        bool _discontinued;

        static Random _rnd = new Random();
        static string[] _names = "Macko|Surfair|Pocohey|Studeby".Split('|');
        static string[] _lines = "Computers|Washers|Stoves|Cars".Split('|');
        static string[] _colors = "Red|Green|Blue|White".Split('|');

        public Product()
        {
            Name = PickOne(_names);
            Line = PickOne(_lines);
            Color = PickOne(_colors);
            Price = 30 + _rnd.NextDouble() * 1000;
            Cost = 3 + _rnd.NextDouble() * 300;
            Discontinued = _rnd.NextDouble() < .2;
            Introduced = DateTime.Today.AddDays(_rnd.Next(-600, 0));
        }

        static public List<Product> GetData(int Count)
        {
            var list = new List<Product>();
            for (int i = 0; i < Count; i++)
            {
                list.Add(new Product());
            }
            return list;
        }

        string PickOne(string[] options)
        {
            return options[_rnd.Next() % options.Length];
        }

        [Display(Name = "Name")]
        public string Name
        {
            get { return _name; }
            set { SetProperty("Name", ref _name, value); }
        }

        [Display(Name = "Color")]
        public string Color
        {
            get { return _color; }
            set { SetProperty("Color", ref _color, value); }
        }

        [Display(Name = "Line")]
        public string Line
        {
            get { return _line; }
            set { SetProperty("Line", ref _line, value); }
        }

        [Display(Name = "Price")]
        public double Price
        {
            get { return _price; }
            set { SetProperty("Price", ref _price, value); }
        }

        [Display(Name = "Cost")]
        public double Cost
        {
            get { return _cost; }
            set { SetProperty("Cost", ref _cost, value); }
        }

        [Display(Name = "Introduced")]
        public DateTime Introduced
        {
            get { return _date; }
            set { SetProperty("Introduced", ref _date, value); }
        }

        [Display(Name = "Discontinued")]
        public bool Discontinued
        {
            get { return _discontinued; }
            set { SetProperty("Discontinued", ref _discontinued, value); }
        }
        [Display(Name = "Special Price")]
        [Editable(true)]
        public double CustomField1 { get; set; }
    }
    public class BaseObject : INotifyPropertyChanged, IEditableObject
    {
        protected bool SetProperty<T>(string propName, ref T field, T value)
        {
            if (EqualityComparer<T>.Default.Equals(field, value)) return false;
            field = value;
            OnPropertyChanged(propName);
            return true;
        }

        #region ** INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        void OnPropertyChanged(string propName)
        {
            OnPropertyChanged(new PropertyChangedEventArgs(propName));
        }
        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, e);
        }

        #endregion

        #region ** IEditableObject

        object _clone = null;
        public void BeginEdit()
        {
            _clone = this.MemberwiseClone();
        }
        public void CancelEdit()
        {
            foreach (var p in this.GetType().GetProperties())
            {
                var value = p.GetValue(_clone, null);
                p.SetValue(this, value, null);
            }
        }
        public void EndEdit()
        {
            _clone = null;
        }

        #endregion
    }
    public class FlexGridEE : C1FlexGrid, ISupportExpressions
    {
        #region Ctor
        public FlexGridEE() : base()
        {
            ExpressionEditors = new ExpressionEditorCollection(this);
        }
        #endregion

        #region Override
        protected override void OnCellEditEnded(CellEditEventArgs e)
        {
            base.OnCellEditEnded(e);
            // ignore expression column changing
            if (!e.Cancel && !ExpressionEditors.Contains(Columns[e.Column].ColumnName))
                ExpressionEditors.Evaluate(GetDataIndex(e.Row));
        }
        #endregion

        #region ISupportExpressions
        public void SetCellValue(int row, string colName, object value)
        {
            this[GetRowIndex(row), colName] = value;
        }

        public ExpressionEditorCollection ExpressionEditors { get; }
        #endregion
    }
    public class ImageCellFactory : CellFactory
    {
        public override void CreateColumnHeaderContent(C1FlexGrid grid, Border bdr, CellRange rng)
        {
            base.CreateColumnHeaderContent(grid, bdr, rng);
            if (grid.Columns[rng.Column].Header == "Special Price")
            {
                string bitmapPath = @"pack://application:,,/Resources/fx.png";
                BitmapImage bitmap = new BitmapImage(new Uri(bitmapPath));
                System.Windows.Controls.Image img = new System.Windows.Controls.Image();
                img.HorizontalAlignment = HorizontalAlignment.Right;
                img.Source = bitmap;
                var tb = bdr.Child;
                bdr.Child = null;
                StackPanel sp = new StackPanel();
                sp.Orientation = Orientation.Horizontal;
                sp.Children.Add(tb);
                sp.Children.Add(img);
                bdr.Child = sp;
                grid.Columns[rng.Column].Width = new GridLength(100, GridUnitType.Pixel);
            }
        }
    }
}
